@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2021 -- leonerd@leonerd.org.uk
 
-package XS::Parse::Keyword 0.04;
+package XS::Parse::Keyword 0.05;
 
 use v5.14;
 use warnings;
@@ -188,15 +188,45 @@ relating some optional part of them is present in the incoming source text. In
 this case, the pieces relating to those optional parts must support "probing".
 This ability is also noted below.
 
-The type of each piece should be one of the following macro values:
+The type of each piece should be one of the following macro values. Some
+macros additionally take a set of typeflags; taken from the following list:
+
+=over 2
+
+=item * XPK_TYPEFLAG_SCOPED
+
+On C<XPK_BLOCK_flags>, this will wrap the returned optree in its own lexical
+scope, by calling C<op_scope()>.
+
+=item * XPK_TYPEFLAG_G_VOID, XPK_TYPEFLAG_G_SCALAR, XPK_TYPEFLAG_G_ARRAY
+
+On optree-returning types, will contextualize the returned optree to put it in
+the given context, by calling C<op_contextualize()>.
+
+=back
+
+Where both a plain and a C<_flags>-suffixed version of the macro exists, the
+flags version will take the flags in an additional argument, and the non-flags
+version will pass zero extra flags.
 
 =head2 XPK_BLOCK
 
 I<atomic, emits op.>
 
+   XPK_BLOCK
+
+   XPK_BLOCK_flags(flags)
+
 A brace-delimited block of code is expected, passed as an optree in the I<op>
 field. This will be parsed as a block within the current function scope.
 
+Permits the flags C<XPK_TYPEFLAG_SCOPED> and C<XPK_TYPEFLAG_G_*>.
+
+=head2 XPK_BLOCK_SCALARCTX, XPK_BLOCK_LISTCTX
+
+Shortcuts for C<XPK_BLOCK_flags()> which wrap a scalar or list-context scope
+around the block.
+
 =head2 XPK_ANONSUB
 
 I<atomic, emits op.>
@@ -209,16 +239,38 @@ field.
 
 I<atomic, emits op.>
 
+   XPK_TERMEXPR
+
+   XPK_TERMEXPR_flags(flags)
+
 A term expression is expected, parsed using C<parse_termexpr()>, and passed as
 an optree in the I<op> field.
 
+Permits the flags C<XPK_TYPEFLAG_G_*>
+
+=head2 XPK_TERMEXPR_SCALARCTX
+
+A shortcut for C<XPK_TERMEXPR_flags()> which puts the expression in scalar
+context.
+
 =head2 XPK_LISTEXPR
 
 I<atomic, emits op.>
 
+   XPK_LISTEXPR
+
+   XPK_LISTEXPR_flags(flags)
+
 A list expression is expected, parsed using C<parse_listexpr()>, and passed as
 an optree in the I<op> field.
 
+Permits the flags C<XPK_TYPEFLAG_G_*>
+
+=head2 XPK_LISTEXPR_LISTCTX
+
+A shortcut for C<XPK_LISTEXPR_flags()> which puts the expression in list
+context.
+
 =head2 XPK_IDENT
 
 I<atomic, emits sv.>
