@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2021 -- leonerd@leonerd.org.uk
 
-package XS::Parse::Keyword::Builder 0.04;
+package XS::Parse::Keyword::Builder 0.05;
 
 use v5.14;
 use warnings;
@@ -173,13 +173,28 @@ enum {
 };
 
 enum {
-  XS_PARSE_KEYWORD_TYPEFLAG_OPT = (1<<16),
+  XPK_TYPEFLAG_OPT    = (1<<16),
+  XPK_TYPEFLAG_SCOPED = (1<<17),
+  /* These three are shifted versions of perl's G_VOID, G_SCALAR, G_ARRAY */
+  XPK_TYPEFLAG_G_VOID   = (1<<18),
+  XPK_TYPEFLAG_G_SCALAR = (2<<18),
+  XPK_TYPEFLAG_G_ARRAY  = (3<<18), /* yes it's called ARRAY, not LIST */
 };
 
-#define XPK_BLOCK    {.type = XS_PARSE_KEYWORD_BLOCK}
-#define XPK_ANONSUB  {.type = XS_PARSE_KEYWORD_ANONSUB}
-#define XPK_TERMEXPR {.type = XS_PARSE_KEYWORD_TERMEXPR}
-#define XPK_LISTEXPR {.type = XS_PARSE_KEYWORD_LISTEXPR}
+#define XPK_BLOCK_flags(flags) {.type = XS_PARSE_KEYWORD_BLOCK|(flags)}
+#define XPK_BLOCK              XPK_BLOCK_flags(0)
+#define XPK_BLOCK_SCALARCTX    XPK_BLOCK_flags(XPK_TYPEFLAG_SCOPED|XPK_TYPEFLAG_G_SCALAR)
+#define XPK_BLOCK_LISTCTX      XPK_BLOCK_flags(XPK_TYPEFLAG_SCOPED|XPK_TYPEFLAG_G_ARRAY)
+
+#define XPK_ANONSUB {.type = XS_PARSE_KEYWORD_ANONSUB}
+
+#define XPK_TERMEXPR_flags(flags) {.type = XS_PARSE_KEYWORD_TERMEXPR|(flags)}
+#define XPK_TERMEXPR              XPK_TERMEXPR_flags(0)
+#define XPK_TERMEXPR_SCALARCTX    XPK_TERMEXPR_flags(XPK_TYPEFLAG_G_SCALAR)
+#define XPK_LISTEXPR_flags(flags) {.type = XS_PARSE_KEYWORD_LISTEXPR|(flags)}
+#define XPK_LISTEXPR              XPK_LISTEXPR_flags(0)
+#define XPK_LISTEXPR_LISTCTX      XPK_LISTEXPR_flags(XPK_TYPEFLAG_G_ARRAY)
+
 #define XPK_IDENT    {.type = XS_PARSE_KEYWORD_IDENT}
 #define XPK_PACKAGENAME {.type = XS_PARSE_KEYWORD_PACKAGENAME}
 
@@ -188,7 +203,7 @@ enum {
 #define XPK_ATTRIBUTES {.type = XS_PARSE_KEYWORD_ATTRS}
 
 #define XPK_VSTRING     {.type = XS_PARSE_KEYWORD_VSTRING}
-#define XPK_VSTRING_OPT {.type = XS_PARSE_KEYWORD_VSTRING|XS_PARSE_KEYWORD_TYPEFLAG_OPT}
+#define XPK_VSTRING_OPT {.type = XS_PARSE_KEYWORD_VSTRING|XPK_TYPEFLAG_OPT}
 
 #define XPK_COLON  {.type = XS_PARSE_KEYWORD_LITERALCHAR, .u.c = ':'}
 #define XPK_EQUALS {.type = XS_PARSE_KEYWORD_LITERALCHAR, .u.c = '='}
